@@ -1,29 +1,29 @@
-asArgs <- function (...)
+.asArgs <- function (x)
 {
-    n <- nargs()
+    n <- length(x)
     if (!n)
         return(character())
     else if (n == 1L) {
-        x <- ..1
-        if (is.null(x))
+        xx <- x[[1L]]
+        if (is.null(xx))
             return(character())
-        else if (is.object(x)) {
+        else if (is.object(xx)) {
         }
-        else if (is.numeric(x) || is.complex(x))
-            return(format.default(x, trim = TRUE, digits = 17L,
+        else if (!length(xx))
+            return(character())
+        else if (is.numeric(xx) || is.complex(xx))
+            return(format.default(xx, trim = TRUE, digits = 17L,
                 decimal.mark = ".", drop0trailing = TRUE))
-        else if (is.raw(x))
-            return(sprintf("0x%02x", as.integer(x)))
-        else if (is.logical(x) || is.character(x)) {
-            value <- as.character(x)
+        else if (is.raw(xx))
+            return(sprintf("0x%02x", as.integer(xx)))
+        else if (is.logical(xx) || is.character(xx)) {
+            value <- as.character(xx)
             if (anyNA(value))
                 value[is.na(value)] <- "NA"
             return(value)
         }
-        else if (!length(x))
-            return(character())
     }
-    value <- rapply(list(...), function(xx) {
+    value <- rapply(x, function(xx) {
         if (is.null(xx))
             character()
         else if (is.object(xx)) {
@@ -35,13 +35,15 @@ asArgs <- function (...)
                 format.POSIXlt(xx, format = "%Y-%m-%d %H:%M:%OS6")
             else as.character(xx)
         }
+        else if (!length(xx))
+            character()
         else if (is.numeric(xx) || is.complex(xx))
             format.default(xx, trim = TRUE, digits = 17L, decimal.mark = ".",
                 drop0trailing = TRUE)
         else if (is.raw(xx))
             sprintf("0x%02x", as.integer(xx))
         else if (is.pairlist(xx))
-            asArgs(as.list(xx))
+            .asArgs(as.vector(xx, "list"))
         else as.character(xx)
     }, how = "replace")
     value <- unlist(value, recursive = TRUE, use.names = FALSE)
@@ -51,27 +53,50 @@ asArgs <- function (...)
 }
 
 
-from.shell <- function ()
-isTRUE(attr(this.path(default = return(FALSE), verbose = FALSE), "this.path::from.shell"))
+asArgs <- function (...)
+.asArgs(list(...))
 
 
 fileArgs <- function ()
 {
-    path <- this.path(default = return(character()), verbose = FALSE)
-    # cat("\nhere\n")
-    if (!is.null(n <- attr(path, "this.path::n")) &&
-        identical(sys.function(n - 1L), withArgs))
-    {
-        get("args", envir = sys.frame(n - 1L), inherits = FALSE)
+    n <- .this.path(get.frame.number = TRUE)
+    if (n) {
+        if (n == 1L) {
+            if (has.shFILE)
+                character()
+            else commandArgs(trailingOnly = TRUE)
+        }
+        else if (identical(sys.function(n - 1L), withArgs))
+            get("args", envir = sys.frame(n - 1L), inherits = FALSE)
+        else character()
     }
-    else if (isTRUE(attr(path, "this.path::from.shell")))
+    else if (has.shFILE)
         commandArgs(trailingOnly = TRUE)
     else character()
 }
 
 
-withArgs <- function (expr, ...)
+progArgs <- function ()
 {
-    args <- asArgs(...)
-    expr
+    n <- .this.path(get.frame.number = TRUE)
+    if (n) {
+        if (n == 1L) {
+            if (has.shFILE)
+                character()
+            else commandArgs(trailingOnly = TRUE)
+        }
+        else if (identical(sys.function(n - 1L), withArgs))
+            get("args", envir = sys.frame(n - 1L), inherits = FALSE)
+        else character()
+    }
+    else if (in.shell)
+        commandArgs(trailingOnly = TRUE)
+    else character()
+}
+
+
+withArgs <- function (...)
+{
+    args <- .External2(C_asargs, 1L)
+    ..1
 }
